package rwth.i2.ltlrv.afastate.base;

import java.util.Set;

import rwth.i2.ltlrv.afastate.interfaze.IAFAState;
import rwth.i2.ltlrv.afastate.interfaze.INullaryAFAState;
import rwth.i2.ltlrv.data.WeakValuesMap;

/**
 * NullaryAFAState - Abstract base class of all nullary AFA states.
 * 
 * @author Eric Bodden
 */
public abstract class NullaryAFAState extends AbstractAFAState implements INullaryAFAState {

    /**
     * {@inheritDoc}
     */
    public IAFAState negationNormalForm() {
        //this is already in negation normal form
        return this;
    }

    /**
     * {@inheritDoc}
     */
    //TODO still needed?
    public void unboundVariablesAtCurrentJoinpoint(@SuppressWarnings("unused") VariableKind variableKind, @SuppressWarnings("unused") Set<String> result) {
        //no unbound variables exist in FF 
    }

    
    /**
     * {@inheritDoc}
     */
    @SuppressWarnings("unused")
    public IAFAState specializeBindings(WeakValuesMap<String, Object> bindings) {
        //nothing to sspecialize
        return this;
    }
    
    /**
     * {@inheritDoc}
     */
    @SuppressWarnings("unused")
    public void providesPos(Set<String> result) {
        //provides nothing
    }
    
    /**
     * {@inheritDoc}
     */
    @SuppressWarnings("unused")
    public void providesNeg(Set<String> result) {
        //provides nothing
    }
    
    /**
     * {@inheritDoc}
     */
    @SuppressWarnings("unused")
    public void requires(Set<String> result) {
    	//requires nothing
    }

    /**
     * {@inheritDoc}
     */
    @Override
    @SuppressWarnings("unused")
    public void validate(Set<String> context) throws ValidationException {
    	//always ok
    }

    /**
     * {@inheritDoc}
     */
    public INullaryAFAState getInstance() {
        return this;
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public String toString() {
        return symbol();
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public int hashCode() {
        return symbol().hashCode();
    }

}
