package rwth.i2.ltlrv.management;

import rwth.i2.ltlrv.afastate.interfaze.IIfClosure;
import rwth.i2.ltlrv.afastate.interfaze.IProposition;
import rwth.i2.ltlrv.data.WeakValuesHashMap;
import rwth.i2.ltlrv.data.WeakValuesMap;


/**
 * EvalIfTestEquality
 * 
 * @author Eric Bodden
 */
@SuppressWarnings("all")
public class EvalIfTestEquality extends EvalAbstractEvaluationTest {

    private final static IIfClosure[] closureAEqB = new IIfClosure[] {
            new IIfClosure() {

                public boolean satisfiedUnderBindings(WeakValuesMap<String, Object> currentBinding) throws UserCausedException {
                    Object a = currentBinding.get("a");
                    Object b = currentBinding.get("b");
                    return a==b;
                }
                
                public String toString() { return "a==b"; }

				public String[] variableNames() {
					return new String[] {"a","b"};
				}
                
            }            
    };
    
    private final static IProposition PROP_A = f.Proposition("p", new String[] {"a"});
    private final static IProposition PROP_B_BeqA = f.Proposition("p", new String[] {"b"}, closureAEqB );
    
    public EvalIfTestEquality() {
        super( f.G(f.Impl(p_a,f.X(f.F(PROP_B_BeqA)))) ); 
    }
    
    public void testEqualityPositiveSameObject() {
        assertTrue(config.isFinal());

        WeakValuesMap<String, Object> bindings = new WeakValuesHashMap<String,Object>();
        bindings.put("a","o1");
        props.add(PROP_A.specializeBindings(bindings));
        config.transition(props);
        assertFalse(config.isFinal());
        
        props.clear();
        
        bindings = new WeakValuesHashMap<String,Object>();
        bindings.put("b","o1");
        props.add(PROP_B_BeqA.specializeBindings(bindings));
        config.transition(props);
        assertTrue(config.isFinal());
    }
    
    public void testEqualityNegativeWrongObject() {
        assertTrue(config.isFinal());

        WeakValuesMap<String, Object> bindings = new WeakValuesHashMap<String,Object>();
        bindings.put("a","o1");
        props.add(PROP_A.specializeBindings(bindings));
        config.transition(props);
        assertFalse(config.isFinal());
        
        props.clear();
        
        bindings = new WeakValuesHashMap<String,Object>();
        bindings.put("b","o2");
        props.add(PROP_B_BeqA.specializeBindings(bindings));
        config.transition(props);
        assertFalse(config.isFinal());
    }

     
}
