package rwth.i2.ltlrv.management;

import rwth.i2.ltlrv.afastate.interfaze.IIfClosure;
import rwth.i2.ltlrv.afastate.interfaze.IProposition;
import rwth.i2.ltlrv.data.WeakValuesHashMap;
import rwth.i2.ltlrv.data.WeakValuesMap;


/**
 * EvalIfTestEqualityRelease - Tests (q(b)(b==a) R p(a))
 * 
 * @author Eric Bodden
 */
@SuppressWarnings("all")
public class EvalIfTestEqualityRelease extends EvalAbstractEvaluationTest {

    private final static IIfClosure[] closureAEqB = new IIfClosure[] {
            new IIfClosure() {

                public boolean satisfiedUnderBindings(WeakValuesMap<String, Object> currentBinding) throws UserCausedException {
                    Object a = currentBinding.get("a");
                    Object b = currentBinding.get("b");
                    return a==b;
                }
                
                public String toString() { return "a==b"; }

				public String[] variableNames() {
					return new String[] {"a","b"};
				}
                
            }            
    };
    
    private final static IProposition P_A = f.Proposition("p", new String[] {"a"});
    private final static IProposition Q_B_BeqA = f.Proposition("q", new String[] {"b"}, closureAEqB );
    
    public EvalIfTestEqualityRelease() {
        super( f.Release(Q_B_BeqA, P_A) ); 
    }
    
    public void testPositive() {
        assertTrue(config.isFinal());

        WeakValuesMap<String, Object> bindings = new WeakValuesHashMap<String,Object>();
        bindings.put("a","o1");
        props.add(P_A.specializeBindings(bindings));
        //p(o1)
        config.transition(props);
        assertTrue(config.isFinal());
        assertFalse(config.isTT());
        
        //p(o1)
        config.transition(props);
        assertTrue(config.isFinal());
        assertFalse(config.isTT());

        bindings.clear();
        bindings.put("b","o1");
        props.add(Q_B_BeqA.specializeBindings(bindings));
        //q(o1),p(o1)
        config.transition(props);
        assertTrue(config.isTT());
    }

    public void testNegativeWrongArgument() {
        assertTrue(config.isFinal());

        WeakValuesMap<String, Object> bindings = new WeakValuesHashMap<String,Object>();
        bindings.put("a","o1");
        props.add(P_A.specializeBindings(bindings));
        //p(o1)
        config.transition(props);
        assertTrue(config.isFinal());
        assertFalse(config.isTT());
        
        //p(o1)
        config.transition(props);
        assertTrue(config.isFinal());
        assertFalse(config.isTT());

        bindings.put("b","o2");
        props.add(Q_B_BeqA.specializeBindings(bindings));
        //q(o2),p(o1)
        config.transition(props);
        assertTrue(config.isFinal());
        assertFalse(config.isTT());

        bindings.clear();
        props.clear();
        bindings.put("b","o2");
        props.add(Q_B_BeqA.specializeBindings(bindings));
        //q(o2)
        config.transition(props);
        assertTrue(config.isFF());        
    }
     
}
