package rwth.i2.ltlrv.management;

import rwth.i2.ltlrv.afastate.interfaze.IIfClosure;
import rwth.i2.ltlrv.afastate.interfaze.IProposition;
import rwth.i2.ltlrv.data.WeakValuesHashMap;
import rwth.i2.ltlrv.data.WeakValuesMap;


/**
 * EvalIfTestEqualityReleaseNext2 - Tests (X(q(b)(b==a)) R p(a,b)). The question is here
 * whether the left 'b' is <i>the</i> 'b' bound by 'p' or whether it is a fresh b'
 * (with private scope). We believe that it is more natural to define it as the former,
 * being bound by p. These tests (in particular testNegativeSemanticViolatio) ensure those semantics.  
 * 
 * @author Eric Bodden
 */
@SuppressWarnings("all")
public class EvalIfTestEqualityReleaseNext2 extends EvalAbstractEvaluationTest {

    private final static IIfClosure[] closureAEqB = new IIfClosure[] {
            new IIfClosure() {

                public boolean satisfiedUnderBindings(WeakValuesMap<String, Object> currentBinding) throws UserCausedException {
                    Object a = currentBinding.get("a");
                    Object b = currentBinding.get("b");
                    return a==b;
                }
                
                public String toString() { return "a==b"; }

				public String[] variableNames() {
					return new String[] {"a","b"};
				}
                
            }            
    };
    
    private final static IProposition P_A_B = f.Proposition("p", new String[] {"a","b"});
    private final static IProposition Q_B_BeqA = f.Proposition("q", new String[] {"b"}, closureAEqB );
    
    public EvalIfTestEqualityReleaseNext2() {
        super( f.Release(f.X(Q_B_BeqA), P_A_B) ); 
    }
    
    public void testPositive() {
        assertTrue(config.isFinal());

        WeakValuesMap<String, Object> bindings = new WeakValuesHashMap<String,Object>();
        bindings.put("a","o1");
        bindings.put("b","o1");
        props.add(P_A_B.specializeBindings(bindings));
        //p(o1,o2)
        config.transition(props);
        assertTrue(config.isFinal());
        assertFalse(config.isTT());
        
        //p(o1,o2)
        config.transition(props);
        assertTrue(config.isFinal());
        assertFalse(config.isTT());

        props.clear();
        bindings.clear();
        
        bindings.put("b","o1");
        props.add(Q_B_BeqA.specializeBindings(bindings));
        //q(o1) 
        config.transition(props);
        //here, q(o1) holds and before we had o1=a=b in p
        assertTrue(config.isTT());
    }
    
    public void testNegativeSemanticViolation() {
        assertTrue(config.isFinal());

        WeakValuesMap<String, Object> bindings = new WeakValuesHashMap<String,Object>();
        bindings.put("a","o1");
        bindings.put("b","o2");
        props.add(P_A_B.specializeBindings(bindings));
        //p(o1,o2)
        config.transition(props);
        assertTrue(config.isFinal());
        assertFalse(config.isTT());
        
        //p(o1,o2)
        config.transition(props);
        assertTrue(config.isFinal());
        assertFalse(config.isTT());

        props.clear();
        bindings.clear();
        
        bindings.put("b","o1");
        props.add(Q_B_BeqA.specializeBindings(bindings));
        //q(o1)
        config.transition(props);
        //according to the semantics, q is already in state q(a=o1,b=o2)[a==b], which can never be true
        assertTrue(config.isFF());
    }

    public void testNegativeWrongProposition() {
        assertTrue(config.isFinal());

        WeakValuesMap<String, Object> bindings = new WeakValuesHashMap<String,Object>();
        bindings.put("a","o1");
        bindings.put("b","o1");
        props.add(P_A_B.specializeBindings(bindings));
        //p(o1,o2)
        config.transition(props);
        assertTrue(config.isFinal());
        assertFalse(config.isTT());
        
        //p(o1,o2)
        config.transition(props);
        assertTrue(config.isFinal());
        assertFalse(config.isTT());

        props.clear();
        bindings.clear();
        
        bindings.put("b","o2");
        props.add(Q_B_BeqA.specializeBindings(bindings));
        //q(o2)
        config.transition(props);
        //here, before we had p(a,b) with a=o1,b=o1, so q(o2) is not the expected proposition here
        assertTrue(config.isFF());
    }
     
}
