package rwth.i2.ltlrv.management;

import rwth.i2.ltlrv.afastate.interfaze.IIfClosure;
import rwth.i2.ltlrv.afastate.interfaze.IProposition;
import rwth.i2.ltlrv.data.WeakValuesHashMap;
import rwth.i2.ltlrv.data.WeakValuesMap;
import rwth.i2.ltlrv.formula.interfaze.IFormula;

/**
 * EvalLOR
 * 
 * @author Eric Bodden
 */
@SuppressWarnings("all")
public class EvalLOR extends EvalAbstractEvaluationTest {

    //////////////////////////////////////////////////////////
    // GENERATED CODE HERE
    //////////////////////////////////////////////////////////
    
    private static final IFormulaFactory factory = new FormulaFactory();

    private final static IProposition lock_i_x = factory.Proposition(
            "entry(call(static void (lor.Main).lock(..)) && args(i, x))",
            new java.lang.String[] { "i", "x" },
            new IIfClosure[] {});

    private final static IProposition unlock_i_y = factory.Proposition(
            "entry(call(static void (lor.Main).unlock(..)) && args(i, y))",
            new java.lang.String[] { "i", "y" },
            new IIfClosure[] {});

    private final static IProposition lock_j_y = factory.Proposition(
            "entry(call(static void (lor.Main).lock(..)) && args(j, y))",
            new java.lang.String[] { "j", "y" },
            new IIfClosure[] {});

    private final static IProposition lock_j_y_j_ue_i = factory
            .Proposition(
                    ("entry(call(static void (lor.Main).lock(..)) && args(j, y) &&"
                            + " if(j != i))"),
                    new java.lang.String[] { "j", "y" },
                    new IIfClosure[] { new IIfClosure() {
                        public boolean satisfiedUnderBindings(
                                WeakValuesMap currentBindings) {
                            try {
                                final Integer j = (Integer) currentBindings
                                        .get("j");
                                final Integer i = (Integer) currentBindings
                                        .get("i");
                                try {
                                    return j != i;
                                } catch (java.lang.Exception ex) {
                                    throw new IIfClosure.UserCausedException(
                                            ex, "j != i");
                                }
                            } catch (java.lang.NullPointerException ex) {
                                return false;
                            } catch (java.lang.ClassCastException ex) {
                                return false;
                            }
                        }

                        public java.lang.String[] variableNames() {
                            return new java.lang.String[] { "j", "i" };
                        }

                        public java.lang.String toString() {
                            return "j != i";
                        }
                    } });

    private final static IProposition lock_i_y_x_ue_y = factory
            .Proposition(
                    ("entry(call(static void (lor.Main).lock(..)) && args(i, y) &&"
                            + " if(x != y))"),
                    new java.lang.String[] { "i", "y" },
                    new IIfClosure[] { new IIfClosure() {
                        public boolean satisfiedUnderBindings(
                                WeakValuesMap currentBindings) {
                            try {
                                final Integer x = (Integer) currentBindings
                                        .get("x");
                                final Integer y = (Integer) currentBindings
                                        .get("y");
                                try {
                                    return x != y;
                                } catch (java.lang.Exception ex) {
                                    throw new IIfClosure.UserCausedException(
                                            ex, "x != y");
                                }
                            } catch (java.lang.NullPointerException ex) {
                                return false;
                            } catch (java.lang.ClassCastException ex) {
                                return false;
                            }
                        }

                        public java.lang.String[] variableNames() {
                            return new java.lang.String[] { "x", "y" };
                        }

                        public java.lang.String toString() {
                            return "x != y";
                        }
                    } });

    private final static IProposition unlock_j_x = factory.Proposition(
            "entry(call(static void (lor.Main).unlock(..)) && args(j, x))",
            new java.lang.String[] { "j", "x" },
            new IIfClosure[] {});

    private final static IProposition lock_j_x_j_uq_i = factory
            .Proposition(
                    ("entry(call(static void (lor.Main).lock(..)) && args(j, x) &&"
                            + " if(j != i))"),
                    new java.lang.String[] { "j", "x" },
                    new IIfClosure[] { new IIfClosure() {
                        public boolean satisfiedUnderBindings(
                                WeakValuesMap currentBindings) {
                            try {
                                final Integer j = (Integer) currentBindings
                                        .get("j");
                                final Integer i = (Integer) currentBindings
                                        .get("i");
                                try {
                                    return j != i;
                                } catch (java.lang.Exception ex) {
                                    throw new IIfClosure.UserCausedException(
                                            ex, "j != i");
                                }
                            } catch (java.lang.NullPointerException ex) {
                                return false;
                            } catch (java.lang.ClassCastException ex) {
                                return false;
                            }
                        }

                        public java.lang.String[] variableNames() {
                            return new java.lang.String[] { "j", "i" };
                        }

                        public java.lang.String toString() {
                            return "j != i";
                        }
                    } });

    //copy of lock_i_x 
    private final static IProposition lock_i_x_1 = factory.Proposition(
            "entry(call(static void (lor.Main).lock(..)) && args(i, x))",
            new java.lang.String[] { "i", "x" },
            new IIfClosure[] {});

    private final static IFormula formula = factory.G(factory.Until(factory
            .Not(lock_i_x_1), factory.And(lock_i_x, factory.X(factory.Impl(factory
            .Until(factory.Not(unlock_i_y), lock_i_y_x_ue_y), factory.G(factory.Not(factory
            .Until(factory.Not(lock_j_y), factory.And(lock_j_y_j_ue_i, factory.X(factory
                    .Until(factory.Not(unlock_j_x), lock_j_x_j_uq_i)))))))))));

    //////////////////////////////////////////////////////////
    // END OF GENERATED CODE
    //////////////////////////////////////////////////////////

    public EvalLOR() {
        super(formula);
    }
    
    public void testInterleavedLocking() {
        assertTrue(config.isFinal());
        assertFalse(config.isTT());
        assertFalse(config.isFF());
        
        lock(1,1);
        
        assertFalse(config.isTT());
        assertFalse(config.isFF());
        assertTrue(config.isFinal());

        lock(2,2);
        
        assertFalse(config.isTT());
        assertFalse(config.isFF());
        assertTrue(config.isFinal());

        lock(1,2);
        
        assertFalse(config.isTT());
        assertFalse(config.isFF());
        assertTrue(config.isFinal());

        lock(2,1);
        
        assertTrue(config.isFF());
    }
    
    public void testUnlock() {
        assertTrue(config.isFinal());
        assertFalse(config.isTT());
        assertFalse(config.isFF());
        
        lock(1,1);
        
        assertFalse(config.isTT());
        assertFalse(config.isFF());
        
        unlock(1,1);
        
        assertFalse(config.isTT());
        assertFalse(config.isFF());

        lock(2,2);
        
        assertFalse(config.isTT());
        assertFalse(config.isFF());

        lock(1,2);
        
        assertFalse(config.isTT());
        assertFalse(config.isFF());

        lock(2,1);
        
        assertFalse(config.isTT());
        assertFalse(config.isFF());
    }    
    
    void lock(int t, int l) {
        props.clear();
        
        WeakValuesMap<String, Object> b_i_x = new WeakValuesHashMap<String, Object>();
        b_i_x.put("i", t);
        b_i_x.put("x", l);
        WeakValuesMap<String, Object> b_i_y = new WeakValuesHashMap<String, Object>();
        b_i_y.put("i", t);
        b_i_y.put("y", l);
        WeakValuesMap<String, Object> b_j_x = new WeakValuesHashMap<String, Object>();
        b_j_x.put("j", t);
        b_j_x.put("x", l);
        WeakValuesMap<String, Object> b_j_y = new WeakValuesHashMap<String, Object>();
        b_j_y.put("j", t);
        b_j_y.put("y", l);

        props.add(lock_i_x.specializeBindings(b_i_x));

        props.add(lock_i_x_1.specializeBindings(b_i_x));
        
        props.add(lock_i_y_x_ue_y.specializeBindings(b_i_y));
        
        props.add(lock_j_x_j_uq_i.specializeBindings(b_j_x));
        
        props.add(lock_j_y.specializeBindings(b_j_y));
        
        props.add(lock_j_y_j_ue_i.specializeBindings(b_j_y));        

        config.transition(props);
    }
    
    void unlock(int t, int l) {
        props.clear();
        
        WeakValuesMap<String, Object> b_j_x = new WeakValuesHashMap<String, Object>();
        b_j_x.put("j", t);
        b_j_x.put("x", l);
        WeakValuesMap<String, Object> b_i_y = new WeakValuesHashMap<String, Object>();
        b_i_y.put("i", t);
        b_i_y.put("y", l);

        props.add(unlock_j_x.specializeBindings(b_j_x));

        props.add(unlock_i_y.specializeBindings(b_i_y));

        config.transition(props);
    }
}
